@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2007 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2008 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,6 +46,8 @@ typedef struct
         GtkWindowClass   parent_class;
 
         /* signals */
+        void (* begin_auto_login)            (GdmGreeterLoginWindow *login_window,
+                                              const char            *username);
         void (* begin_verification)          (GdmGreeterLoginWindow *login_window);
         void (* begin_verification_for_user) (GdmGreeterLoginWindow *login_window,
                                               const char            *username);
@@ -52,8 +55,10 @@ typedef struct
                                               const char            *text);
         void (* user_selected)               (GdmGreeterLoginWindow *login_window,
                                               const char            *text);
+        void (* session_selected)            (GdmGreeterLoginWindow *login_window,
+                                              const char            *text);
         void (* cancelled)                   (GdmGreeterLoginWindow *login_window);
-        void (* disconnected)                (GdmGreeterLoginWindow *login_window);
+        void (* start_session)               (GdmGreeterLoginWindow *login_window);
 
 } GdmGreeterLoginWindowClass;
 
@@ -62,6 +67,7 @@ GtkWidget *         gdm_greeter_login_window_new                (gboolean displa
 
 
 gboolean            gdm_greeter_login_window_reset              (GdmGreeterLoginWindow *login_window);
+gboolean            gdm_greeter_login_window_authentication_failed (GdmGreeterLoginWindow *login_window);
 gboolean            gdm_greeter_login_window_ready              (GdmGreeterLoginWindow *login_window);
 gboolean            gdm_greeter_login_window_info_query         (GdmGreeterLoginWindow *login_window,
                                                                  const char *text);
@@ -71,6 +77,13 @@ gboolean            gdm_greeter_login_window_info               (GdmGreeterLogin
                                                                  const char *text);
 gboolean            gdm_greeter_login_window_problem            (GdmGreeterLoginWindow *login_window,
                                                                  const char *text);
+void                gdm_greeter_login_window_set_default_session_name (GdmGreeterLoginWindow *login_window,
+                                                                       const char *text);
+
+void               gdm_greeter_login_window_request_timed_login (GdmGreeterLoginWindow *login_window,
+                                                                 const char            *username,
+                                                                 int                    delay);
+void               gdm_greeter_login_window_user_authorized     (GdmGreeterLoginWindow *login_window);
 
 G_END_DECLS
 
